-------------------------------------------------------------------------------
--! @file       HS1_SIV_pkg.vhd
--! @brief      Package for HS1_SIV module
--! @project    CAESAR Candidate Evaluation
--! @author     Ekawat (ice) Homsirikamol
--! @copyright  Copyright (c) 2016 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package HS1_SIV_pkg is
    constant ZEROS      : std_logic_vector(128              -1 downto 0)
                            := (others => '0');
    constant SIGMA      : std_logic_vector(128              -1 downto 0)
                            := x"617078653320646E79622D326B206574";
    constant CONS_STR   : std_logic_vector( 96              -1 downto 0)
                            := x"001000100040040C00000000";
    
    function switch_endian_byte (
        x       : std_logic_vector;
        w       : integer)
    return std_logic_vector;
end HS1_SIV_pkg;

package body HS1_SIV_pkg is

    function switch_endian_byte (
        x       : std_logic_vector;
        w       : integer)
    return std_logic_vector is
		variable retval : std_logic_vector(x'high downto x'low);
	begin
		for i in 0 to x'length/w-1 loop
			for j in 0 to w/8-1 loop
				retval(x'high-i*w-j*8 downto x'high-i*w-j*8-7)
                    := x(x'high-i*w-(w/8-j-1)*8
                            downto x'high-i*w-(w/8-j-1)*8-7);
			end loop;
		end loop;
		return(retval);
	end switch_endian_byte;

end package body;