-------------------------------------------------------------------------------
--! @file       AEAD_TP_TB_Wrapper.vhd
--! @brief      AEAD Two-Pass Wrapper for Testbench.
--!             This file should be used if targeting a two-pass algorithm.
--! @project    CAESAR Candidate Evaluation
--! @author     Ekawat (ice) Homsirikamol
--! @copyright  Copyright (c) 2015 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity AEAD is
    generic (
        G_ASYNC_RSTN         : boolean := False;
        G_W                  : integer := 64;
        G_SW                 : integer := 32;
        G_FW                 : integer := 128
    );
    port (
        --! Global signals
        clk                  : in  std_logic;
        rst                  : in  std_logic;

        --! Data in signals
        pdi_data             : in  std_logic_vector(G_W         -1 downto 0);
        pdi_valid            : in  std_logic;
        pdi_ready            : out std_logic;

        --! Key signals
        sdi_data             : in  std_logic_vector(G_SW        -1 downto 0);
        sdi_valid            : in  std_logic;
        sdi_ready            : out std_logic;

        --! Data out signals
        do_data              : out std_logic_vector(G_W         -1 downto 0);
        do_ready             : in  std_logic;
        do_valid             : out std_logic
    );
end AEAD;

architecture structure of AEAD is
    signal   fdi_data       : std_logic_vector(G_FW-1 downto 0);
    signal   fdo_data       : std_logic_vector(G_FW-1 downto 0);
    signal   fdi_valid      : std_logic;
    signal   fdi_ready      : std_logic;
    signal   fdo_valid      : std_logic;
    signal   fdo_ready      : std_logic;
begin
    u_logic:
    entity work.AEAD_TP(structure)
    generic map (
        G_ASYNC_RSTN        => G_ASYNC_RSTN ,
        G_W                 => G_W          ,
        G_SW                => G_SW         ,
        G_FW                => G_FW
    )
    port map (
        clk                 => clk          ,
        rst                 => rst          ,
        pdi_data            => pdi_data     ,
        pdi_valid           => pdi_valid    ,
        pdi_ready           => pdi_ready    ,
        sdi_data            => sdi_data     ,
        sdi_valid           => sdi_valid    ,
        sdi_ready           => sdi_ready    ,
        do_data             => do_data      ,
        do_ready            => do_ready     ,
        do_valid            => do_valid     ,

        --! FIFO signals
        fdi_data            => fdi_data     ,
        fdi_valid           => fdi_valid    ,
        fdi_ready           => fdi_ready    ,
        fdo_data            => fdo_data     ,
        fdo_valid           => fdo_valid    ,
        fdo_ready           => fdo_ready
    );

    u_twopass_fifo:
    entity work.fwft_fifo(structure)
    generic map (
        G_W             => G_FW             ,
        G_LOG2DEPTH     => 11               ,
        G_ASYNC_RSTN    => G_ASYNC_RSTN
    )
    port map (
        clk             => clk              ,
        rst             => rst              ,
        din             => fdo_data         ,
        din_valid       => fdo_valid        ,
        din_ready       => fdo_ready        ,
        dout            => fdi_data         ,
        dout_valid      => fdi_valid        ,
        dout_ready      => fdi_ready
    );
end structure;